Patches to 1.1.11 to add support for the iBCS emulator. (These are known
to work with 1.1.13 as well)

These patches are largely the same as the 1.0 patches. All should apply
cleanly (but with fuzz in some cases) but don't be surprised if you
have to hand edit some things if you apply them to a kernel later
than 1.1.11. If you are about to apply them to a kernel less than 1.1.11
then *don't*. They won't apply cleanly and it may not be obvious what
to do. Besides, the 1.1 kernel is the alpha developers' version. If you
are using it you should keep it up to date as critical bugs are likely
to come and go regularly.

This moves the iBCS stuff clear of the standard kernel but allows the
iBCS module to be loaded by a standard kernel.

This patch file is annotated. If you need to apply this you ought to
read it first!

This patch file creates some new header files. To get them in the
correct place you should be using 'patch -p0' or 'patch -p1' if you
are in the linux source directory already.


===========================================================================

    ELF/COFF support is bundled together as IBCS support now. If you say
    yes to this question you'll get the iBCS emulator linked in to your
    kernel (assuming you have replaced the stub ibcs directory with the
    real one). If you say no (which I recommend) you'll get kernel hooks
    for the loadable module form of the iBCS emulator regardless of whether
    you have the full emulator or not. If you have replaced the stub ibcs
    directory with the real one you'll also get the iBCS module built for you.

===========================================================================

diff -u linux/config.in:1.1.1.1 linux/config.in:1.1.1.1.8.1
--- linux/config.in:1.1.1.1	Tue Mar 15 17:05:25 1994
+++ linux/config.in	Tue Mar 15 17:05:25 1994
@@ -15,8 +15,7 @@
 
 comment 'Program binary formats'
 
-bool 'Elf executables' CONFIG_BINFMT_ELF n
-bool 'COFF executables' CONFIG_BINFMT_COFF n
+bool 'Link with Elf/COFF support' CONFIG_BINFMT_IBCS n
 
 comment 'SCSI support'
 


============================================================================

    The ELF and COFF loaders are now in the (full) ibcs directory. If you
    don't have the full ibcs emulator you don't need these. You should
    delete the binfmt_elf.c and binfmt_coff.c files from the fs directory
    since these are no further use right now.

============================================================================

diff -u linux/fs/Makefile:1.1.1.1 linux/fs/Makefile:1.1.1.1.8.1
--- linux/fs/Makefile:1.1.1.1	Tue Mar 15 17:05:30 1994
+++ linux/fs/Makefile	Tue Mar 15 17:05:30 1994
@@ -40,12 +40,6 @@
 FS_SUBDIRS := $(FS_SUBDIRS) hpfs
 endif
 
-ifdef CONFIG_BINFMT_ELF
-BINFMTS := $(BINFMTS) binfmt_elf.o
-endif
-ifdef CONFIG_BINFMT_COFF
-BINFMTS := $(BINFMTS) binfmt_coff.o
-endif
 
 .c.s:
 	$(CC) $(CFLAGS) -S $<


===========================================================================

    The flag that indicates an ELF executable becomes a more general
    purpose code that indicates the "personality" of an executable
    allowing us to correctly emulate different sets of system calls
    (SVR3 vendors tend to extend the base SVR3 set with their own
    without following any particular standard - symlinks support is
    a common problem). The "personality" can also be used to emulate
    lower level kernel "features". <evil laugh>

============================================================================

diff -u linux/include/linux/sched.h:1.1.1.1 linux/include/linux/sched.h:1.1.1.1.8.1
--- linux/include/linux/sched.h:1.1.1.1	Tue Mar 15 17:05:36 1994
+++ linux/include/linux/sched.h	Tue Mar 15 17:05:36 1994
@@ -180,7 +180,7 @@
 	unsigned long saved_kernel_stack;
 	unsigned long kernel_stack_page;
 	int exit_code, exit_signal;
-	int elf_executable:1;
+	unsigned long personality;
 	int dumpable:1;
 	int swappable:1;
 	int did_exec:1;
diff -u /dev/null linux/include/linux/personality.h:1.1.2.1
--- /dev/null	Tue Mar 15 17:05:36 1994
+++ linux/include/linux/personality.h	Tue Mar 15 17:05:36 1994
@@ -0,0 +1,12 @@
+/* Flags for bug emulation. These occupy the top three bytes. */
+#define STICKY_TIMEOUTS		0x8000000
+#define WHOLE_SECONDS		0x4000000
+
+/* Personality types. These go in the low byte. */
+#define PER_MASK		(0x00ff)
+#define PER_LINUX		(0x0000)
+#define PER_SVR4		(0x0001 | STICKY_TIMEOUTS)
+#define PER_SVR3		(0x0002 | STICKY_TIMEOUTS)
+#define PER_SCOSVR3		(0x0003 | STICKY_TIMEOUTS | WHOLE_SECONDS)
+#define PER_WYSEV386		(0x0004 | STICKY_TIMEOUTS)
+#define PER_ISCR4		(0x0005 | STICKY_TIMEOUTS)


===========================================================================

    For the standard (Linux) exec we just default the personality to
    zero. The ELF and COFF loaders (which are now in the full ibcs
    directory) default to SVR4 and SVR3 respectively.

    We also leave slots in the table of loaders so that we can hook
    in the ELF, COFF and XOUT loaders when the iBCS module is loaded.

============================================================================

diff -u linux/fs/exec.c:1.1.1.1 linux/fs/exec.c:1.1.1.1.8.1
--- linux/fs/exec.c:1.1.1.1	Tue Mar 15 17:05:31 1994
+++ linux/fs/exec.c	Wed Mar 23 09:38:31 1994
@@ -530,13 +530,13 @@
 	if (last_task_used_math == current)
 		last_task_used_math = NULL;
 	current->used_math = 0;
-	current->elf_executable = 0;
+	current->personality = 0;
 }
 
 /*
  * sys_execve() executes a new program.
  */
-static int do_execve(char * filename, char ** argv, char ** envp, struct pt_regs * regs)
+int do_execve(char * filename, char ** argv, char ** envp, struct pt_regs * regs)
 {
 	struct linux_binprm bprm;
 	struct linux_binfmt * fmt;
@@ -730,26 +730,34 @@
 			    struct pt_regs * regs);
 extern int load_aout_library(int fd);
 
-#ifdef CONFIG_BINFMT_ELF
+#ifdef CONFIG_BINFMT_IBCS
+/* These come from the loaders in the linux/ibcs directory. */
 extern int load_elf_binary(struct linux_binprm *,
 			    struct pt_regs * regs);
 extern int load_elf_library(int fd);
-#endif
 
-#ifdef CONFIG_BINFMT_COFF
 extern int load_coff_binary(struct linux_binprm *,
 			    struct pt_regs * regs);
 extern int load_coff_library(int fd);
+
+extern int load_xout_binary(struct linux_binprm *,
+			    struct pt_regs * regs);
+extern int load_xout_library(int fd);
 #endif
 
 /* Here are the actual binaries that will be accepted  */
 struct linux_binfmt formats[] = {
 	{load_aout_binary, load_aout_library},
-#ifdef CONFIG_BINFMT_ELF
+#ifdef CONFIG_BINFMT_IBCS
 	{load_elf_binary, load_elf_library},
-#endif
-#ifdef CONFIG_BINFMT_COFF
 	{load_coff_binary, load_coff_library},
+	{load_xout_binary, load_xout_library},
+#endif
+#ifndef CONFIG_BINFMT_IBCS
+	/* Not compiled in so leave slots for the module to be loaded. */
+	{NULL, NULL},
+	{NULL, NULL},
+	{NULL, NULL},
 #endif
 	{NULL, NULL}
 };


===========================================================================

  Ok, here's the demo kernel "feature" :-). Some systems expect the
  timeout value in a select to be left unchanged. Currently this is
  the only kernel "feature" emulated. Others may be necessary?

============================================================================

diff -u linux/fs/select.c:1.1.1.1 linux/fs/select.c:1.1.1.1.8.1
--- linux/fs/select.c:1.1.1.1	Tue Mar 15 17:05:31 1994
+++ linux/fs/select.c	Tue Mar 15 17:05:31 1994
@@ -3,6 +3,11 @@
  *
  * Created for Linux based loosely upon Mathius Lattner's minix
  * patches by Peter MacDonald. Heavily edited by Linus.
+ *
+ *  4 February 1994
+ *     COFF/ELF binary emulation. If the process has the STICKY_TIMEOUTS
+ *     flag set in its personality we do *not* modify the given timeout
+ *     parameter to reflect time remaining.
  */
 
 #include <linux/types.h>
@@ -14,6 +19,7 @@
 #include <linux/stat.h>
 #include <linux/signal.h>
 #include <linux/errno.h>
+#include <linux/personality.h>
 
 #include <asm/segment.h>
 #include <asm/system.h>
@@ -235,7 +241,7 @@
 	else
 		timeout = 0;
 	current->timeout = 0;
-	if (tvp) {
+	if (tvp && !(current->personality & STICKY_TIMEOUTS)) {
 		put_fs_long(timeout/HZ, (unsigned long *) &tvp->tv_sec);
 		timeout %= HZ;
 		timeout *= (1000000/HZ);


============================================================================

    Change the stub ibcs emulator so it has a hook for the loadable
    module to attach to. The kernel will be capable of loading the
    iBCS module even if only the stub ibcs code was compiled.

============================================================================

diff -u linux/ibcs/emulate.c:1.1.1.1 linux/ibcs/emulate.c:1.1.1.1.8.1
--- linux/ibcs/emulate.c:1.1.1.1	Tue Mar 15 17:05:33 1994
+++ linux/ibcs/emulate.c	Tue Mar 15 17:05:34 1994
@@ -20,8 +20,14 @@
 #include <asm/segment.h>
 #include <asm/system.h>
 
+void (* iABI_hook)(struct pt_regs * regs) = NULL;
+
 asmlinkage void iABI_emulate(struct pt_regs * regs)
 {
-	printk("iBCS2 binaries not supported yet\n");
-	do_exit(SIGSEGV);
+	if (iABI_hook) {
+		iABI_hook(regs);
+	} else {
+		printk(KERN_INFO "iBCS: emulation module not loaded\n");
+		do_exit(SIGSEGV);
+	}
 }


============================================================================

    Add the symbols that the loadable iBCS module requires to the table
    used by the modules code.

============================================================================

diff -u -r1.1 linux/kernel/ksyms.c
--- 1.1	1994/04/08 08:48:00
+++ linux/kernel/ksyms.c	1994/04/08 11:39:19
@@ -24,7 +24,7 @@
 extern void (*do_floppy)(void);
 #endif
 
-#ifdef CONFIG_BINFMT_IBCS
+#ifndef CONFIG_BINFMT_IBCS
 extern int do_execve(char * filename, char ** argv, char ** envp,
 		struct pt_regs * regs);
 extern void flush_old_exec(struct linux_binprm * bprm);
@@ -103,7 +103,7 @@
 	X(do_floppy),
 #endif
 
-#ifdef CONFIG_BINFMT_IBCS
+#ifndef CONFIG_BINFMT_IBCS
 /*
  * The following are needed if iBCS support is modular rather than
  * compiled in.
 

=============================================================================

    We also need to change the signal routines slightly to give us an
    iBCS stack frame.

============================================================================

diff -u linux/kernel/signal.c:1.1.1.1 linux/kernel/signal.c:1.1.1.1.8.1
--- linux/kernel/signal.c:1.1.1.1	Tue Mar 15 17:05:38 1994
+++ linux/kernel/signal.c	Tue Mar 15 17:05:38 1994
@@ -137,7 +137,7 @@
  * isn't actually ignored, but does automatic child reaping, while
  * SIG_DFL is explicitly said by POSIX to force the signal to be ignored..
  */
-static void check_pending(int signum)
+void check_pending(int signum)
 {
 	struct sigaction *p;
 
@@ -248,6 +248,8 @@
 	do_exit(SIGSEGV);
 }
 
+int (*ibcs_invmapsig)(int) = NULL;
+
 /*
  * Set up a signal frame... Make the stack look the way iBCS2 expects
  * it to look.
@@ -267,7 +269,10 @@
 		do_exit(SIGSEGV);
 /* set up the "normal" stack seen by the signal handler (iBCS2) */
 	put_fs_long(__CODE,frame);
-	put_fs_long(signr, frame+1);
+	if(current->personality && ibcs_invmapsig)
+	  put_fs_long(ibcs_invmapsig(signr), frame+1);
+	else
+	  put_fs_long(signr, frame+1);
 	put_fs_long(regs->gs, frame+2);
 	put_fs_long(regs->fs, frame+3);
 	put_fs_long(regs->es, frame+4);


=============================================================================

    The table of file systems is now a list in 1.1.8 but the pointer
    to the head of the list needs to be global not static.

============================================================================

diff -u -r1.1.1.1.14.1 linux/fs/super.c
--- 1.1.1.1.14.1	1994/04/22 09:27:52
+++ linux/fs/super.c	1994/04/22 11:06:03
@@ -35,7 +35,7 @@
 /* this is initialized in init/main.c */
 dev_t ROOT_DEV = 0;
 
-static struct file_system_type * file_systems = NULL;
+struct file_system_type * file_systems = NULL;
 
 int register_filesystem(struct file_system_type * fs)
 {



That's all, folks!

(You actually *read* this then?)
