/*
 * little utility to change run time options for iBCS
 *
 *  Copyright (C) 1999  Mike Jagdis (jaggy@purplet.demon.co.uk)
 *
 * $Id: options.c,v 1.1 1999/01/14 22:37:18 jaggy Exp $
 * $Source: /nfs4/sophia/home/mjagdis/src/ibcs.cvs/ibcs/Tools/options.c,v $
 */
#define _GNU_SOURCE
#include <features.h>

#include <linux/personality.h>
#include <stdio.h>
#include <stdlib.h>

#if __GLIBC__ >= 2
#  include <syscall.h>
#  include <unistd.h>
#else
#  include <linux/unistd.h>
#endif

#include <ibcs/options.h>


#if __GLIBC__ >= 2
#  define personality(X)	syscall(__NR_personality, (X))
#else
   _syscall1(int, personality, int, pers);
#endif


#ifdef __sparc__
static int
set_options(int val)
{
	int res;

	__asm__ volatile ("mov %1,%%o0\n\t"
			  "mov 195,%%g1\n\t"
			  "t 8\n\t"
			  "mov %%o0,%0" : "=r" (res) : "r" (val) : "o0", "g1");
        return res;
}

#else /* __sparc__ */

static int
set_options(int val)
{
	int res;

	__asm__ volatile ("mov\t$0x02FF,%%eax\n\t"
		".byte\t0x9a,0,0,0,0,7,0\n\t"
		: "=a" (res));

	return res;
}


#endif /* __sparc__ */


struct o_code {
	char	*name;
	int	code;
};

struct o_code codes[] = {
	{ "sco-id-sco",		OPTION_SCO_ID_SCO	},
	{ "sco-id-os5",		OPTION_SCO_ID_OS5	}
};


static int
get_code(char *s)
{
	int i;

	for (i=0; i<sizeof(codes)/sizeof(codes[0]); i++)
		if (!strcmp(s, codes[i].name))
			return codes[i].code;

	fprintf(stderr, "%s not recognised - ignored\n", s);
	return 0;
}


static void
usage(void)
{
	int i;

	fprintf(stderr,
		"usage: trace n         - set options to n\n"
		"          or [+|-]code - where code could be:\n"
		"                  query\n"
		"                  off\n");

	for (i=0; i<sizeof(codes)/sizeof(codes[0]); i++)
		fprintf(stderr, "                  %s\n", codes[i].name);
}


int
main(int argc, char *argv[])
{
	int old_options_val, options_val, i;

	if (argc == 1) {
		usage();
		exit(1);
	}

	/* We need to be in a non-native personality to get access to
	 * the emulator's trace facilities.
	 */
	personality(PER_SVR3);

	old_options_val = options_val = set_options(-1);
	for (i=1; i<argc; i++) {
		if (!strcmp(argv[i], "query")) {
			printf("iBCS options code is 0x%lx\n", set_options(-1));
		} else if (!strcmp(argv[i], "off")) {
			options_val = 0;
		} else if (argv[i][0] == '+') {
			options_val |= get_code(argv[i]+1);
		} else if (argv[i][0] == '-') {
			options_val &= (~get_code(argv[i]+1));
		} else {
			options_val |= get_code(argv[i]);
		}
	}

	if (options_val != old_options_val)
		printf("iBCS options code is 0x%lx\n",
			set_options(options_val));

	exit(0);
}
