#include <errno.h>
#include <signal.h>
#include <stdio.h>

#include "ldt.h"
#include "syscall.h"
#include "lcall7.h"

#ifdef DEBUG
# include "debug.h"
#endif


int
emu_read(struct sigcontext_struct *sc)
{
#ifdef DEBUG
	if (__dbf)
		fprintf(__dbf, "x286emul:   file = %d, buf = 0x%x:0x%04lx, length = 0x%04x\n",
			(sc->ebx & 0xffff),
			sc->ds, (sc->ecx & 0xffff), (sc->esi & 0xffff));
#endif
	return read(sc->ebx & 0xffff,
		ldt[sc->ds>>3].base + (sc->ecx & 0xffff),
		sc->esi & 0xffff);
}


int
emu_write(struct sigcontext_struct *sc)
{
#ifdef DEBUG
	if (__dbf)
		fprintf(__dbf, "x286emul:   file = %d, buf = 0x%x:0x%04lx, length = 0x%04x\n",
			(sc->ebx & 0xffff),
			sc->ds, (sc->ecx & 0xffff), (sc->esi & 0xffff));
#endif
	return write(sc->ebx & 0xffff,
		ldt[sc->ds>>3].base + (sc->ecx & 0xffff),
		sc->esi & 0xffff);
}


int
emu_open(struct sigcontext_struct *sc)
{
#ifdef DEBUG
	if (__dbf)
		fprintf(__dbf, "x286emul:   file: 0x%x:0x%04x, \"%s\", flags: 0x%04lx, mode: 0x%04x\n",
			sc->ds, (sc->ebx & 0xffff),
			ldt[sc->ds>>3].base + (sc->ebx & 0xffff),
			(sc->ecx & 0xffff), (sc->esi & 0xffff));
#endif
	return lcall7(sc->eax & 0xffff,
		ldt[sc->ds>>3].base + (sc->ebx & 0xffff),
		(sc->ecx & 0xffff),
		(sc->esi & 0xffff));
}


int
emu_close(struct sigcontext_struct *sc)
{
	return close((sc->ebx & 0xffff));
}


int
emu_lseek(struct sigcontext_struct *sc)
{
	return lseek((sc->ebx & 0xffff),
			(sc->ecx & 0xffff) | (sc->esi << 16),
			(sc->edi & 0xffff));
}


int
emu_unlink(struct sigcontext_struct *sc)
{
#ifdef DEBUG
	fprintf(__dbf, "x286emul:   path = 0x%x:0x%04lx \"%s\"\n",
		sc->ds, (sc->ebx & 0xffff),
		ldt[sc->ds>>3].base + (sc->ebx & 0xffff));
#endif
	return unlink(ldt[sc->ds>>3].base + (sc->ebx & 0xffff));
}


int
emu_ioctl(struct sigcontext_struct *sc)
{
#ifdef DEBUG
	if (__dbf)
		fprintf(__dbf, "x286emul:   fd: %d, op: 0x%04lx, addr: 0x%x:0x%04x\n",
			(sc->ebx & 0xffff),
			(sc->ecx & 0xffff),
			sc->ds, ldt[sc->ds>>3].base + (sc->esi & 0xffff));
#endif
	return lcall7(sc->eax & 0xffff,
		(sc->ebx & 0xffff),
		(sc->ecx & 0xffff),
		ldt[sc->ds>>3].base + (sc->esi & 0xffff));
}
